package objects

import "git.tebibyte.media/tomo/tomo"
import "git.tebibyte.media/tomo/tomo/input"
import "git.tebibyte.media/tomo/objects/layouts"

var _ tomo.Object = new(Notebook)

// Notebook holds multiple objects, each in their own tab. The user can click
// the tab bar at the top to choose which one is activated.
//
// Sub-components:
//   - TabRow sits at the top of the container and contains a row of tabs.
//   - TabSpacer sits at either end of the tab row, bookending the list of tabs.
//   - Tab appears in the tab row for each tab in the container. The user can
//     click on the tab to switch to it.
//
// TabSpacer tags:
//   - [left] The spacer is on the left.
//   - [right] The spacer is on the right.
//
// Tab tags:
//   - [active] The tab is currently active and its contents are visible.
type Notebook struct {
	box tomo.ContainerBox
	
	leftSpacer  tomo.Box
	rightSpacer tomo.Box
	tabsRow tomo.ContainerBox
	active string
	tabs []*tab
}

// NewNotebook creates a new tabbed notebook.
func NewNotebook () *Notebook {
	notebook := &Notebook {
		box: tomo.NewContainerBox(),
	}
	notebook.box.SetRole(tomo.R("objects", "Notebook"))
	notebook.box.SetAttr(tomo.ALayout(layouts.Column { false, true }))
	
	notebook.tabsRow = tomo.NewContainerBox()
	notebook.tabsRow.SetRole(tomo.R("objects", "TabRow"))
	notebook.box.Add(notebook.tabsRow)
	
	notebook.leftSpacer = tomo.NewBox()
	notebook.leftSpacer.SetRole(tomo.R("objects", "TabSpacer"))
	notebook.leftSpacer.SetTag("left", true)
	notebook.rightSpacer = tomo.NewBox()
	notebook.rightSpacer.SetRole(tomo.R("objects", "TabSpacer"))
	notebook.rightSpacer.SetTag("right", true)
	
	notebook.Clear()
	notebook.setTabRowLayout()
	return notebook
}

// GetBox returns the underlying box.
func (this *Notebook) GetBox () tomo.Box {
	return this.box
}

// Activate switches to a named tab.
func (this *Notebook) Activate (name string) {
	if _, tab := this.findTab(this.active); tab != nil {
		tab.setActive(false)
		this.box.Remove(tab.root)
	}
	if _, tab := this.findTab(name); tab != nil {
		tab.setActive(true)
		this.box.Add(tab.root)
	} else {
		name = ""
	}
	this.active = name
}

// Add adds an object as a tab with the specified name.
func (this *Notebook) Add (name string, root tomo.Object) {
	tab := &tab {
		TextBox: tomo.NewTextBox(),
		name:    name,
		root:    root,
	}
	tab.SetRole(tomo.R("objects", "Tab"))
	tab.SetText(name)
	tab.OnButtonDown(func (button input.Button) bool {
		if button != input.ButtonLeft { return false }
		this.Activate(name)
		return true
	})
	tab.OnButtonUp(func (button input.Button) bool {
		if button != input.ButtonLeft { return false }
		return true
	})
	
	this.tabs = append(this.tabs, tab)
	this.tabsRow.Insert(tab, this.rightSpacer)
	this.setTabRowLayout()
	
	// if the row was empty before, activate this tab
	if len(this.tabs) == 1 {
		this.Activate(name)	
	}
}

// Remove removes the named tab.
func (this *Notebook) Remove (name string) {
	index, tab := this.findTab(name)
	if index < 0 { return }
	nextIndex := index - 1
	
	this.tabsRow.Remove(tab)
	this.tabs = append(this.tabs[:index], this.tabs[index - 1:]...)
	this.setTabRowLayout()
	
	if nextIndex < 0               { nextIndex = 0 }
	if nextIndex >= len(this.tabs) { nextIndex = len(this.tabs) - 1 }
	if nextIndex < 0 {
		this.Activate("")
	} else {
		this.Activate(this.tabs[nextIndex].name)
	}
}

// Clear removes all tabs.
func (this *Notebook) Clear () {
	this.tabs = nil
	this.tabsRow.Clear()
	this.tabsRow.Add(this.leftSpacer)
	this.tabsRow.Add(this.rightSpacer)
}

func (this *Notebook) setTabRowLayout () {
	row := make(layouts.Row, 1 + len(this.tabs) + 1)
	row[len(row) - 1] = true
	this.tabsRow.SetAttr(tomo.ALayout(row))
}

func (this *Notebook) findTab (name string) (int, *tab) {
	for index, tab := range this.tabs {
		if tab.name == name { return index, tab }
	}
	return -1, nil
}

type tab struct {
	tomo.TextBox
	name string
	root tomo.Object
}

func (this *tab) setActive (active bool) {
	if active {
		this.SetRole(tomo.R("objects", "Tab"))
		this.SetTag("active", true)
	} else {
		this.SetRole(tomo.R("objects", "Tab"))
		this.SetTag("active", false)
	}
}

